class NodeNotFound(Exception):
    pass


class PortNotFound(Exception):
    pass


class LinkNotFound(Exception):
    pass


class TopologyNotFound(Exception):
    pass


class PortIdInUse(Exception):
    pass


class LinkIdInUse(Exception):
    pass


class PathIdInUse(Exception):
    pass


class NodeIdInUse(Exception):
    pass


class NumericNodeId(Exception):
    pass
